class Langs:
    """
    ISO 639-1 Language Codes
    ISO 639-1 defines abbreviations for languages:
    """

    Abkhazian = "ab"
    Afar = "aa"
    Afrikaans = "af"
    Akan = "ak"
    Albanian = "sq"
    Amharic = "am"
    Arabic = "ar"
    Aragonese = "an"
    Armenian = "hy"
    Assamese = "as"
    Avaric = "av"
    Avestan = "ae"
    Aymara = "ay"
    Azerbaijani = "az"
    Bambara = "bm"
    Bashkir = "ba"
    Basque = "eu"
    Belarusian = "be"
    Bengali_Bangla = "bn"
    Bihari = "bh"
    Bislama = "bi"
    Bosnian = "bs"
    Breton = "br"
    Bulgarian = "bg"
    Burmese = "my"
    Catalan = "ca"
    Chamorro = "ch"
    Chechen = "ce"
    Chichewa_Chewa_Nyanja = "ny"
    Chinese = "zh"
    Chinese_Simplified = "zh-Hans"
    Chinese_Traditional = "zh-Hant"
    Chuvash = "cv"
    Cornish = "kw"
    Corsican = "co"
    Cree = "cr"
    Croatian = "hr"
    Czech = "cs"
    Danish = "da"
    Divehi_Dhivehi_Maldivian = "dv"
    Dutch = "nl"
    Dzongkha = "dz"
    English = "en"
    Esperanto = "eo"
    Estonian = "et"
    Ewe = "ee"
    Faroese = "fo"
    Fijian = "fj"
    Finnish = "fi"
    French = "fr"
    Fula_Fulah_Pulaar_Pular = "ff"
    Galician = "gl"
    Gaelic_Scottish = "gd"
    Gaelic_Manx = "gv"
    Georgian = "ka"
    German = "de"
    Greek = "el"
    Greenlandic = "kl"
    Guarani = "gn"
    Gujarati = "gu"
    Haitian_Creole = "ht"
    Hausa = "ha"
    Hebrew = "he"
    Herero = "hz"
    Hindi = "hi"
    Hiri_Motu = "ho"
    Hungarian = "hu"
    Icelandic = "is"
    Ido = "io"
    Igbo = "ig"
    Indonesian = "id in"
    Interlingua = "ia"
    Interlingue = "ie"
    Inuktitut = "iu"
    Inupiak = "ik"
    Irish = "ga"
    Italian = "it"
    Japanese = "ja"
    Javanese = "jv"
    Kalaallisut_Greenlandic = "kl"
    Kannada = "kn"
    Kanuri = "kr"
    Kashmiri = "ks"
    Kazakh = "kk"
    Khmer = "km"
    Kikuyu = "ki"
    Kinyarwanda_Rwanda = "rw"
    Kirundi = "rn"
    Kyrgyz = "ky"
    Komi = "kv"
    Kongo = "kg"
    Korean = "ko"
    Kurdish = "ku"
    Kwanyama = "kj"
    Lao = "lo"
    Latin = "la"
    Latvian_Lettish = "lv"
    Limburgish__Limburger = "li"
    Lingala = "ln"
    Lithuanian = "lt"
    Luga_Katanga = "lu"
    Luganda_Ganda = "lg"
    Luxembourgish = "lb"
    Manx = "gv"
    Macedonian = "mk"
    Malagasy = "mg"
    Malay = "ms"
    Malayalam = "ml"
    Maltese = "mt"
    Maori = "mi"
    Marathi = "mr"
    Marshallese = "mh"
    Moldavian = "mo"
    Mongolian = "mn"
    Nauru = "na"
    Navajo = "nv"
    Ndonga = "ng"
    Northern_Ndebele = "nd"
    Nepali = "ne"
    Norwegian = "no"
    Norwegian_bokm = "nb"
    Norwegian_nynorsk = "nn"
    Nuosu = "ii"
    Occitan = "oc"
    Ojibwe = "oj"
    Old_Church_Slavonic_Old_Bulgarian = "cu"
    Oriya = "or"
    Oromo_Afaan_Oromo = "om"
    Ossetian = "os"
    Pali = "pi"
    Pashto_Pushto = "ps"
    Persian = "fa"
    Farsi = "fa"
    Polish = "pl"
    Portuguese = "pt"
    Punjabi_Eastern = "pa"
    Quechua = "qu"
    Romansh = "rm"
    Romanian = "ro"
    Russian = "ru"
    Sami = "se"
    Samoan = "sm"
    Sango = "sg"
    Sanskrit = "sa"
    Serbian = "sr"
    Serbo_Croatian = "sh"
    Sesotho = "st"
    Setswana = "tn"
    Shona = "sn"
    Sichuan_Yi = "ii"
    Sindhi = "sd"
    Sinhalese = "si"
    Siswati = "ss"
    Slovak = "sk"
    Slovenian = "sl"
    Somali = "so"
    Southern_Ndebele = "nr"
    Spanish = "es"
    Sundanese = "su"
    Swahili_Kiswahili = "sw"
    Swati = "ss"
    Swedish = "sv"
    Tagalog = "tl"
    Tahitian = "ty"
    Tajik = "tg"
    Tamil = "ta"
    Tatar = "tt"
    Telugu = "te"
    Thai = "th"
    Tibetan = "bo"
    Tigrinya = "ti"
    Tonga = "to"
    Tsonga = "ts"
    Turkish = "tr"
    Turkmen = "tk"
    Twi = "tw"
    Uyghur = "ug"
    Ukrainian = "uk"
    Urdu = "ur"
    Uzbek = "uz"
    Venda = "ve"
    Vietnamese = "vi"
    Volap = "vo"
    Wallon = "wa"
    Welsh = "cy"
    Wolof = "wo"
    Western_Frisian = "fy"
    Xhosa = "xh"
    Yiddish = "yi ji"
    Yoruba = "yo"
    Zhuang_Chuang = "za"
    Zulu = "zu"



from inui.base import BaseSvgElement


class A(BaseSvgElement): ...


class Animate(BaseSvgElement): ...


class Animatemotion(BaseSvgElement): ...


class Animatetransform(BaseSvgElement): ...


class Circle(BaseSvgElement): ...


class Clippath(BaseSvgElement): ...


class Cursor(BaseSvgElement): ...


class Defs(BaseSvgElement): ...


class Desc(BaseSvgElement): ...


class Ellipse(BaseSvgElement): ...


class Feblend(BaseSvgElement): ...


class Fecolormatrix(BaseSvgElement): ...


class Fecomponenttransfer(BaseSvgElement): ...


class Fecomposite(BaseSvgElement): ...


class Feconvolvematrix(BaseSvgElement): ...


class Fediffuselighting(BaseSvgElement): ...


class Fedisplacementmap(BaseSvgElement): ...


class Fedistantlight(BaseSvgElement): ...


class Fedropshadow(BaseSvgElement): ...


class Feflood(BaseSvgElement): ...


class Fefunca(BaseSvgElement): ...


class Fefuncb(BaseSvgElement): ...


class Fefuncg(BaseSvgElement): ...


class Fefuncr(BaseSvgElement): ...


class Fegaussianblur(BaseSvgElement): ...


class Feimage(BaseSvgElement): ...


class Femerge(BaseSvgElement): ...


class Femergenode(BaseSvgElement): ...


class Femorphology(BaseSvgElement): ...


class Feoffset(BaseSvgElement): ...


class Fepointlight(BaseSvgElement): ...


class Fespecularlighting(BaseSvgElement): ...


class Fespotlight(BaseSvgElement): ...


class Fetile(BaseSvgElement): ...


class Feturbulence(BaseSvgElement): ...


class Filter(BaseSvgElement): ...


class Font(BaseSvgElement): ...


class Font_face(BaseSvgElement): ...


class Font_face_format(BaseSvgElement): ...


class Font_face_name(BaseSvgElement): ...


class Font_face_src(BaseSvgElement): ...


class Font_face_uri(BaseSvgElement): ...


class Foreignobject(BaseSvgElement): ...


class G(BaseSvgElement): ...


class Glyph(BaseSvgElement): ...


class Glyphref(BaseSvgElement): ...


class Hkern(BaseSvgElement): ...


class Image(BaseSvgElement): ...


class Line(BaseSvgElement): ...


class Lineargradient(BaseSvgElement): ...


class Marker(BaseSvgElement): ...


class Mask(BaseSvgElement): ...


class Metadata(BaseSvgElement): ...


class Missing_glyph(BaseSvgElement): ...


class Mpath(BaseSvgElement): ...


class Path(BaseSvgElement): ...


class Pattern(BaseSvgElement): ...


class Polygon(BaseSvgElement): ...


class Polyline(BaseSvgElement): ...


class Radialgradient(BaseSvgElement): ...


class Rect(BaseSvgElement): ...


class ScriptSvg(BaseSvgElement): ...


class Set(BaseSvgElement): ...


class Stop(BaseSvgElement): ...


class Style(BaseSvgElement): ...


class Svg(BaseSvgElement): ...


class Switch(BaseSvgElement): ...


class Symbol(BaseSvgElement): ...


class Text(BaseSvgElement): ...


class Textpath(BaseSvgElement): ...


class Title(BaseSvgElement): ...


class Tref(BaseSvgElement): ...


class Tspan(BaseSvgElement): ...


class Use(BaseSvgElement): ...


class View(BaseSvgElement): ...


class Vkern(BaseSvgElement): ...



class Colors:
    """
        Color Names Supported by All Browsers
    All modern browsers support the following 140 color names
    (click on a color name, or a hex value, to view the color as the background-color along with different text colors):

        Returns:
            str: Color Hex Code

        functions:
            rgb_to_hex((r,g,b))  --> #RRGGBB
            hex_to_rgb(hex) --> (r,g,b)

    """

    AliceBlue = "#F0F8FF"
    AntiqueWhite = "#FAEBD7"
    Aqua = "#00FFFF"
    Aquamarine = "#7FFFD4"
    Azure = "#F0FFFF"
    Beige = "#F5F5DC"
    Bisque = "#FFE4C4"
    Black = "#000000"
    BlanchedAlmond = "#FFEBCD"
    Blue = "#0000FF"
    BlueViolet = "#8A2BE2"
    Brown = "#A52A2A"
    BurlyWood = "#DEB887"
    CadetBlue = "#5F9EA0"
    Chartreuse = "#7FFF00"
    Chocolate = "#D2691E"
    Coral = "#FF7F50"
    CornflowerBlue = "#6495ED"
    Cornsilk = "#FFF8DC"
    Crimson = "#DC143C"
    Cyan = "#00FFFF"
    DarkBlue = "#00008B"
    DarkCyan = "#008B8B"
    DarkGoldenRod = "#B8860B"
    DarkGray = "#A9A9A9"
    DarkGrey = "#A9A9A9"
    DarkGreen = "#006400"
    DarkKhaki = "#BDB76B"
    DarkMagenta = "#8B008B"
    DarkOliveGreen = "#556B2F"
    DarkOrange = "#FF8C00"
    DarkOrchid = "#9932CC"
    DarkRed = "#8B0000"
    DarkSalmon = "#E9967A"
    DarkSeaGreen = "#8FBC8F"
    DarkSlateBlue = "#483D8B"
    DarkSlateGray = "#2F4F4F"
    DarkSlateGrey = "#2F4F4F"
    DarkTurquoise = "#00CED1"
    DarkViolet = "#9400D3"
    DeepPink = "#FF1493"
    DeepSkyBlue = "#00BFFF"
    DimGray = "#696969"
    DimGrey = "#696969"
    DodgerBlue = "#1E90FF"
    FireBrick = "#B22222"
    FloralWhite = "#FFFAF0"
    ForestGreen = "#228B22"
    Fuchsia = "#FF00FF"
    Gainsboro = "#DCDCDC"
    GhostWhite = "#F8F8FF"
    Gold = "#FFD700"
    GoldenRod = "#DAA520"
    Gray = "#808080"
    Grey = "#808080"
    Green = "#008000"
    GreenYellow = "#ADFF2F"
    HoneyDew = "#F0FFF0"
    HotPink = "#FF69B4"
    IndianRed = "#CD5C5C"
    Indigo = "#4B0082"
    Ivory = "#FFFFF0"
    Khaki = "#F0E68C"
    Lavender = "#E6E6FA"
    LavenderBlush = "#FFF0F5"
    LawnGreen = "#7CFC00"
    LemonChiffon = "#FFFACD"
    LightBlue = "#ADD8E6"
    LightCoral = "#F08080"
    LightCyan = "#E0FFFF"
    LightGoldenRodYellow = "#FAFAD2"
    LightGray = "#D3D3D3"
    LightGrey = "#D3D3D3"
    LightGreen = "#90EE90"
    LightPink = "#FFB6C1"
    LightSalmon = "#FFA07A"
    LightSeaGreen = "#20B2AA"
    LightSkyBlue = "#87CEFA"
    LightSlateGray = "#778899"
    LightSlateGrey = "#778899"
    LightSteelBlue = "#B0C4DE"
    LightYellow = "#FFFFE0"
    Lime = "#00FF00"
    LimeGreen = "#32CD32"
    Linen = "#FAF0E6"
    Magenta = "#FF00FF"
    Maroon = "#800000"
    MediumAquaMarine = "#66CDAA"
    MediumBlue = "#0000CD"
    MediumOrchid = "#BA55D3"
    MediumPurple = "#9370DB"
    MediumSeaGreen = "#3CB371"
    MediumSlateBlue = "#7B68EE"
    MediumSpringGreen = "#00FA9A"
    MediumTurquoise = "#48D1CC"
    MediumVioletRed = "#C71585"
    MidnightBlue = "#191970"
    MintCream = "#F5FFFA"
    MistyRose = "#FFE4E1"
    Moccasin = "#FFE4B5"
    NavajoWhite = "#FFDEAD"
    Navy = "#000080"
    OldLace = "#FDF5E6"
    Olive = "#808000"
    OliveDrab = "#6B8E23"
    Orange = "#FFA500"
    OrangeRed = "#FF4500"
    Orchid = "#DA70D6"
    PaleGoldenRod = "#EEE8AA"
    PaleGreen = "#98FB98"
    PaleTurquoise = "#AFEEEE"
    PaleVioletRed = "#DB7093"
    PapayaWhip = "#FFEFD5"
    PeachPuff = "#FFDAB9"
    Peru = "#CD853F"
    Pink = "#FFC0CB"
    Plum = "#DDA0DD"
    PowderBlue = "#B0E0E6"
    Purple = "#800080"
    RebeccaPurple = "#663399"
    Red = "#FF0000"
    RosyBrown = "#BC8F8F"
    RoyalBlue = "#4169E1"
    SaddleBrown = "#8B4513"
    Salmon = "#FA8072"
    SandyBrown = "#F4A460"
    SeaGreen = "#2E8B57"
    SeaShell = "#FFF5EE"
    Sienna = "#A0522D"
    Silver = "#C0C0C0"
    SkyBlue = "#87CEEB"
    SlateBlue = "#6A5ACD"
    SlateGray = "#708090"
    SlateGrey = "#708090"
    Snow = "#FFFAFA"
    SpringGreen = "#00FF7F"
    SteelBlue = "#4682B4"
    Tan = "#D2B48C"
    Teal = "#008080"
    Thistle = "#D8BFD8"
    Tomato = "#FF6347"
    Turquoise = "#40E0D0"
    Violet = "#EE82EE"
    Wheat = "#F5DEB3"
    White = "#FFFFFF"
    WhiteSmoke = "#F5F5F5"
    Yellow = "#FFFF00"
    YellowGreen = "#9ACD32"

    @staticmethod
    def rgb_to_hex(rgb_tuple):
        """
        Converts an RGB color represented as a tuple (r, g, b) into a hex color code.
        """
        hex_color = "#{:02X}{:02X}{:02X}".format(*rgb_tuple)
        return hex_color

    @staticmethod
    def hex_to_rgb(hex_color):
        """
        Converts a hex color code into an RGB color represented as a tuple (r, g, b).
        """
        hex_color = hex_color.lstrip("#")
        r = int(hex_color[0:2], 16)
        g = int(hex_color[2:4], 16)
        b = int(hex_color[4:6], 16)
        return r, g, b



BOOTSTRAP_VERSION = "5.3.3"

BOOTSTRAP_CSS_URL = (
    "https://cdn.jsdelivr.net/npm/bootstrap@%s/dist/css/bootstrap.min.css"
    % BOOTSTRAP_VERSION
)
BOOTSTRAP_JS_URL = (
    "https://cdn.jsdelivr.net/npm/bootstrap@%s/dist/js/bootstrap.bundle.min.js"
    % BOOTSTRAP_VERSION
)

TAILWINDCSS_URL = "https://cdn.tailwindcss.com"

TAILWINDCSS_CONFIG = {}

TAILWINDCSS_CUSTOM_CSS = ""



from typing import Any

from inui.base import BaseElement, BaseVoidElement


class Doctype(BaseVoidElement): ...


class Abbr(BaseElement): ...


class Acronym(BaseElement): ...


class Address(BaseElement): ...


class A(BaseElement): ...


class Applet(BaseElement): ...


class Area(BaseVoidElement): ...


class Article(BaseElement): ...


class Aside(BaseElement): ...


class Audio(BaseElement): ...


class Base(BaseVoidElement): ...


class Basefont(BaseVoidElement): ...


class Bdi(BaseElement): ...


class Bdo(BaseElement): ...


class Bgsound(BaseVoidElement): ...


class Big(BaseElement): ...


class Blockquote(BaseElement): ...


class Body(BaseElement): ...


class B(BaseElement): ...


class Br(BaseVoidElement): ...


class Button(BaseElement): ...


class Caption(BaseElement): ...


class Canvas(BaseElement): ...


class Center(BaseElement): ...


class Cite(BaseElement): ...


class Code(BaseElement): ...


class Colgroup(BaseElement): ...


class Col(BaseVoidElement): ...


class Comment(BaseElement):
    class Meta:
        start_tag = "!--"
        end_tag = "--"

    def render(self, prettify: bool = False) -> str:
        return f"<{self.start_tag}{' ' if len(self.attributes) > 0 else ''}{str(self.attributes)} {str(self.data)}{self.end_tag}>"


class Data(BaseElement): ...


class Datalist(BaseElement): ...


class Dd(BaseElement): ...


class Dfn(BaseElement): ...


class Del(BaseElement): ...


class Details(BaseElement): ...


class Dialog(BaseElement): ...


class Dir(BaseElement): ...


class Div(BaseElement): ...


class Dl(BaseElement): ...


class Embed(BaseVoidElement): ...


class Fieldset(BaseElement): ...


class Figcaption(BaseElement): ...


class Figure(BaseElement): ...


class Font(BaseElement): ...


class Footer(BaseElement): ...


class Form(BaseElement): ...


class Frame(BaseVoidElement): ...


class Frameset(BaseElement): ...


class Head(BaseElement): ...


class Header(BaseElement): ...


class H1(BaseElement): ...


class H2(BaseElement): ...


class H3(BaseElement): ...


class H4(BaseElement): ...


class H5(BaseElement): ...


class H6(BaseElement): ...


class Hgroup(BaseElement): ...


class Hr(BaseVoidElement): ...


class Html(BaseElement): ...


class Iframe(BaseElement): ...


class Img(BaseElement): ...


class Input(BaseVoidElement): ...


class Ins(BaseElement): ...


class Isindex(BaseVoidElement): ...


class I(BaseElement):  # noqa: E742
    ...


class Kbd(BaseElement): ...


class Keygen(BaseVoidElement): ...


class Label(BaseElement): ...


class Legend(BaseElement): ...


class Li(BaseElement): ...


class Link(BaseElement): ...


class Main(BaseElement): ...


class Mark(BaseElement): ...


class Marquee(BaseElement): ...


class Menuitem(BaseElement): ...


class Meta(BaseVoidElement): ...


class Meter(BaseElement): ...


class Nav(BaseElement): ...


class Nobr(BaseElement): ...


class Noembed(BaseElement): ...


class Noscript(BaseElement): ...


class Object(BaseElement): ...


class Ol(BaseElement): ...


class Optgroup(BaseElement): ...


class Option(BaseElement): ...


class Output(BaseElement): ...


class P(BaseElement): ...


class Pre(BaseElement): ...


class Progress(BaseElement): ...


class Q(BaseElement): ...


class Rp(BaseElement): ...


class Rt(BaseElement): ...


class Ruby(BaseElement): ...


class S(BaseElement): ...


class Samp(BaseElement): ...


class Script(BaseElement): ...


class Section(BaseElement): ...


class Small(BaseElement): ...


class Source(BaseElement): ...


class Spacer(BaseVoidElement): ...


class Span(BaseElement): ...


class Strike(BaseElement): ...


class Strong(BaseElement): ...


class Style(BaseElement): ...


class Sub(BaseElement): ...


class Sup(BaseElement): ...


class Summary(BaseElement): ...


class Svg(BaseElement): ...


class Table(BaseElement): ...


class Tbody(BaseElement): ...


class Td(BaseElement): ...


class Template(BaseElement): ...


class Tfoot(BaseElement): ...


class Th(BaseElement): ...


class Thead(BaseElement): ...


class Time(BaseElement): ...


class Title(BaseElement): ...


class Tr(BaseElement): ...


class Track(BaseElement): ...


class Tt(BaseElement): ...


class U(BaseElement): ...


class Var(BaseElement): ...


class Video(BaseElement): ...


class Wbr(BaseElement): ...


class Xmp(BaseElement): ...
"""
<html>
    <head>
        <title>Hot Reload Example</title>
        <script>
            var ws = new WebSocket("ws://localhost:8000/ws");
            ws.onmessage = function(event) {
                document.body.innerHTML = event.data;
            };
        </script>
    </head>
    <body>
        <h1>Hot Reload</h1>
    </body>
</html>
"""


class HotReload(BaseVoidElement):
    def __init__(
        self, *data: Any, host: str = "localhost", port: int = 8000, **attributes: Any
    ) -> None:
        self.host = host
        self.port = port
        super().__init__(*data, **attributes)

    def render(self, prettify: bool = False):
        return Script(
            """
            var ws = new WebSocket("ws://%s:%s/ws");
            ws.onopen = function() {
                console.log("Connected to WebSocket");
            };
            ws.onmessage = function(event) {
                console.log("Message received:", event.data);
                document.body.innerHTML = event.data;
            };
            ws.onclose = function() {
                console.log("Disconnected from WebSocket");
            };
            """
            % (self.host, str(self.port))
        ).render()


if __name__ == "__main__":
    print(
        Html(
            Head(Title("Hello World")),
            Body(
                "salam",
                Hr(),
                "INUI",
            ),
            Footer("Goodbye World"),
        )
    )



import codecs
import os

from setuptools import find_packages, setup

here = os.path.abspath(os.path.dirname(__file__))
with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()


VERSION = "0.9.6"

PACKAGE_NAME = "inui"
DESCRIPTION = ""
LONG_DESCRIPTION = "Powerful and Highly Customizable Python Library for UI"
AUTHOR_NAME = "Mohammadreza Amani"
AUTHOR_EMAIL = "more.amani@yahoo.com"
PROJECT_URL = "https://github.com/MohammadrezaAmani/INUI/"
REQUIRED_PACKAGES = []
PROJECT_KEYWORDS = ["ui", "python", "html", "framework", "frontend", "bootstrap"]

CLASSIFIERS = [
    "Development Status :: 1 - Planning",
    "Intended Audience :: Developers",
    "Programming Language :: Python :: 3",
    "Operating System :: Unix",
    "Operating System :: MacOS :: MacOS X",
    "Operating System :: Microsoft :: Windows",
]
setup(
    name=PACKAGE_NAME,
    version=VERSION,
    author=AUTHOR_NAME,
    author_email=AUTHOR_EMAIL,
    description=DESCRIPTION,
    url=PROJECT_URL,
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(),
    install_requires=REQUIRED_PACKAGES,
    keywords=PROJECT_KEYWORDS,
    classifiers=CLASSIFIERS,
    entry_points={
        "console_scripts": [
            "inui=inui.hotreload:main",
        ],
    },
)



import re

try:
    from lxml import etree
except ImportError:
    raise "you must install lxml, run `python3 -m pip install lxml"

from inui.config.replaces import reverse_replace


class Element:
    def __init__(self, name="", children=None, attributes=None, data=None):
        self.name = reverse_replace(name).capitalize()
        self.children = children if children is not None else []
        self.attributes = attributes if attributes is not None else {}
        self.data = data

    def add_child(self, child):
        self.children.append(child)

    def __str__(self, indent=0):
        element_str = ""
        if indent == 0:
            element_str += "from inui.elements import *\nfrom inui.svg import *\n"
        attind = " " * (indent + 4)
        attributes_str = "\n".join(
            [
                f"{attind}{reverse_replace(attr)}='''{value}''',"
                for attr, value in self.attributes.items()
            ]
        )
        element_str += f"{' ' * indent}{self.name}(\n{attributes_str}"

        # element_str = f"{' ' * indent}"
        if self.data or self.children:
            element_str += "\n" + attind
        if self.data:
            if self.data.strip():
                # print(self.data)
                element_str += f"'''{self.data}'''," if self.data else ""
        for child in self.children:
            if isinstance(child, str):
                # print(type(child))
                element_str += "\n'''" + child.__str__(indent + 8) + ", '''"
            else:
                element_str += "\n" + child.__str__(indent + 8) + ", "

        element_str += f"\n{' ' * indent})"
        while "''''''" in element_str:
            element_str = element_str.replace("''''''", "''")
        while "''''" in element_str:
            element_str = element_str.replace("''''", "'''")
        return element_str


def create_element_tree(html_text):
    root = etree.HTML(html_text)
    return create_element_from_etree(root)


def create_element_from_etree(element):
    attributes = {}
    if element.attrib:
        attributes = element.attrib

    children = []
    for child in element:
        children.append(create_element_from_etree(child))

    data = element.text if element.text else None

    return Element(
        name=element.tag, children=children, attributes=attributes, data=data
    )


class HtmlTo_inui:
    def __init__(
        self, html=None, url=None, fileName=None, saveTo=None, indent=4
    ) -> None:
        self.html = html
        self.url = url
        self.filename = fileName
        self.indent = indent
        self.inui = None

    def to_inui(self, html_text):
        root = etree.HTML(self._remove_html_comments(html_text))
        return self._create_element_from_etree(root)

    def _remove_html_comments(self, html):
        pattern = r"<!--(.*?)-->"
        clean_html = re.sub(pattern, "", html, flags=re.DOTALL)
        return clean_html

    def _create_element_from_etree(self, element):
        attributes = {}
        if element.attrib:
            attributes = element.attrib

        children = []
        for child in element:
            children.append(self._create_element_from_etree(child))

        data = element.text if element.text else None

        self.inui = Element(
            name=element.tag, children=children, attributes=attributes, data=data
        )
        return self.inui

    def url(self, url):
        try:
            import requests
        except ModuleNotFoundError:
            raise ModuleNotFoundError(
                'requests package not found, try: "python3 -m pip install requests"'
            )
        self.url = url
        self.html = requests.get(self.url).text
        return self.to_inui(self.html)

    def file(self, path=None):
        if path is not None:
            self.filename = path
        with open(self.filename, "r", encoding="utf-8") as f:
            self.html = f.read()
        return self.to_inui(self.html)

    def save(self, path):
        if self.inui:
            self.to_inui(self.html)
        with open(path, "w", encoding="utf-8") as f:
            f.write(str(self.inui))
        return True
